@@ -14,7 +14,7 @@
  * You should have received a copy of the GNU General Public License
  * along with YAD. If not, see <http://www.gnu.org/licenses/>.
  *
- * Copyright (C) 2008-2016, Victor Ananjevsky <ananasik@gmail.com>
+ * Copyright (C) 2008-2017, Victor Ananjevsky <ananasik@gmail.com>
  */
 
 #include "yad.h"
@@ -26,7 +26,7 @@ static void
 entry_activate_cb (GtkEntry * entry, gpointer data)
 {
   if (options.plug == -1)
-    yad_exit (YAD_RESPONSE_OK);
+    yad_exit (options.data.def_resp);
 }
 
 static gboolean
@@ -39,7 +39,7 @@ combo_activate_cb (GtkWidget * w, GdkEventKey * ev, gpointer data)
 #endif
     {
       if (options.plug == -1)
-        yad_exit (YAD_RESPONSE_OK);
+        yad_exit (options.data.def_resp);
       return TRUE;
     }
   return FALSE;
@@ -147,14 +147,18 @@ entry_create_widget (GtkWidget * dlg)
         {
           min = g_ascii_strtod (options.extra_data[0], NULL);
           if (options.extra_data[1])
-            max = g_ascii_strtod (options.extra_data[1], NULL);
-          if (options.extra_data[2])
-            step = g_ascii_strtod (options.extra_data[2], NULL);
-          if (options.extra_data[3])
             {
-              prec = (guint) g_ascii_strtoull (options.extra_data[3], NULL, 0);
-              if (prec > 20)
-                prec = 20;
+              max = g_ascii_strtod (options.extra_data[1], NULL);
+              if (options.extra_data[2])
+                {
+                  step = g_ascii_strtod (options.extra_data[2], NULL);
+                  if (options.extra_data[3])
+                    {
+                      prec = (guint) g_ascii_strtoull (options.extra_data[3], NULL, 0);
+                      if (prec > 20)
+                        prec = 20;
+                    }
+                }
             }
         }
 
