@@ -14,7 +14,7 @@
  * You should have received a copy of the GNU General Public License
  * along with YAD. If not, see <http://www.gnu.org/licenses/>.
  *
- * Copyright (C) 2008-2016, Victor Ananjevsky <ananasik@gmail.com>
+ * Copyright (C) 2008-2017, Victor Ananjevsky <ananasik@gmail.com>
  */
 
 #include "yad.h"
@@ -174,6 +174,15 @@ handle_stdin (GIOChannel * channel, GIOCondition condition, gpointer data)
               return FALSE;
             }
 
+          /* clear list if ^L received */
+          if (string->str[0] == '\014')
+            {
+              gtk_list_store_clear (GTK_LIST_STORE (model));
+              row_count = 0;
+              column_count = 1;
+              continue;
+            }
+
           if (column_count == NUM_COLS)
             {
               /* We're starting a new row */
