@@ -52,7 +52,7 @@ sa_usr1 (gint sig)
   if (options.plug != -1)
     yad_print_result ();
   else
-    yad_exit (YAD_RESPONSE_OK);
+    yad_exit (options.data.def_resp);
 }
 
 static void
@@ -78,7 +78,9 @@ keys_cb (GtkWidget *w, GdkEventKey *ev, gpointer d)
 #else
     case GDK_Escape:
 #endif
-      if (!options.data.no_escape)
+      if (options.data.escape_ok)
+          yad_exit (options.data.def_resp);
+      else if (!options.data.no_escape)
          yad_exit (YAD_RESPONSE_ESC);
       return TRUE;
 #if GTK_CHECK_VERSION(2,24,0)
@@ -367,16 +369,6 @@ create_dialog (void)
   g_signal_connect (G_OBJECT (dlg), "key-press-event", G_CALLBACK (keys_cb), NULL);
   g_signal_connect (G_OBJECT (dlg), "focus-out-event", G_CALLBACK (unfocus_cb), NULL);
 
-#ifndef  G_OS_WIN32
-  /* FIXME: is that very useful !? */
-  if (options.parent)
-    {
-      gdk_window_set_transient_for (gtk_widget_get_window (dlg),
-                                    gdk_x11_window_foreign_new_for_display (gdk_display_get_default (),
-                                                                            options.parent));
-    }
-#endif
-
   /* set window icon */
   if (options.data.window_icon)
     {
@@ -657,8 +649,22 @@ create_dialog (void)
   /* print xid */
   if (options.print_xid)
     {
-      fprintf (stderr, "0x%lX", GDK_WINDOW_XID (gtk_widget_get_window (dlg)));
-      fflush (stderr);
+      FILE *xf;
+
+      if (options.xid_file)
+        xf = fopen (options.xid_file, "w");
+      else
+        xf = stderr;
+
+      if (xf)
+        {
+          fprintf (xf, "0x%lX", GDK_WINDOW_XID (gtk_widget_get_window (dlg)));
+
+          if (options.xid_file)
+            fclose (xf);
+          else
+            fflush (xf);
+        }
     }
 #endif
 
@@ -905,6 +911,18 @@ main (gint argc, gchar ** argv)
       g_print ("%s (GTK+ %d.%d.%d)\n", VERSION, gtk_major_version, gtk_minor_version, gtk_micro_version);
       break;
 
+#ifdef HAVE_SPELL
+    case YAD_MODE_LANGS:
+      show_langs ();
+      break;
+#endif
+
+#ifdef HAVE_SOURCEVIEW
+    case YAD_MODE_THEMES:
+      show_themes ();
+      break;
+#endif
+
     case YAD_MODE_NOTIFICATION:
       ret = yad_notification_run ();
       break;
