@@ -14,7 +14,7 @@
  * You should have received a copy of the GNU General Public License
  * along with YAD. If not, see <http://www.gnu.org/licenses/>.
  *
- * Copyright (C) 2008-2016, Victor Ananjevsky <ananasik@gmail.com>
+ * Copyright (C) 2008-2017, Victor Ananjevsky <ananasik@gmail.com>
  */
 
 #include <stdio.h>
@@ -173,7 +173,7 @@ handle_stdin (GIOChannel * channel, GIOCondition condition, gpointer data)
                         }
 
                       if (need_close && close)
-                        yad_exit (YAD_RESPONSE_OK);
+                        yad_exit (options.data.def_resp);
                     }
                 }
             }
