@@ -14,7 +14,7 @@
  * You should have received a copy of the GNU General Public License
  * along with YAD. If not, see <http://www.gnu.org/licenses/>.
  *
- * Copyright (C) 2008-2016, Victor Ananjevsky <ananasik@gmail.com>
+ * Copyright (C) 2008-2017, Victor Ananjevsky <ananasik@gmail.com>
  */
 
 #include <stdlib.h>
@@ -46,15 +46,25 @@ static gboolean set_size (const gchar *, const gchar *, gpointer, GError **);
 static gboolean set_posx (const gchar *, const gchar *, gpointer, GError **);
 static gboolean set_posy (const gchar *, const gchar *, gpointer, GError **);
 #ifndef G_OS_WIN32
+static gboolean set_xid_file (const gchar *, const gchar *, gpointer, GError **);
 static gboolean parse_signal (const gchar *, const gchar *, gpointer, GError **);
 #endif
 static gboolean add_image_path (const gchar *, const gchar *, gpointer, GError **);
 static gboolean set_complete_type (const gchar *, const gchar *, gpointer, GError **);
 static gboolean set_grid_lines (const gchar *, const gchar *, gpointer, GError **);
 static gboolean set_scroll_policy (const gchar *, const gchar *, gpointer, GError **);
+#if GLIB_CHECK_VERSION(2,30,0)
+static gboolean set_size_format (const gchar *, const gchar *, gpointer, GError **);
+#endif
 
 static gboolean about_mode = FALSE;
 static gboolean version_mode = FALSE;
+#ifdef HAVE_SPELL
+static gboolean langs_mode = FALSE;
+#endif
+#ifdef HAVE_SOURCEVIEW
+static gboolean themes_mode = FALSE;
+#endif
 static gboolean calendar_mode = FALSE;
 static gboolean color_mode = FALSE;
 static gboolean dnd_mode = FALSE;
@@ -107,7 +117,7 @@ static GOptionEntry general_options[] = {
   { "icon-theme", 0, G_OPTION_FLAG_NOALIAS, G_OPTION_ARG_STRING, &options.data.icon_theme,
     N_("Use specified icon theme instead of default"), N_("THEME") },
   { "expander", 0, G_OPTION_FLAG_OPTIONAL_ARG, G_OPTION_ARG_CALLBACK, set_expander,
-    N_("Hide main widget with expander"), N_("TEXT") },
+    N_("Hide main widget with expander"), N_("[TEXT]") },
   { "button", 0, 0, G_OPTION_ARG_CALLBACK, add_button,
     N_("Add dialog button (may be used multiple times)"), N_("NAME:ID") },
   { "no-buttons", 0, 0, G_OPTION_ARG_NONE, &options.data.no_buttons,
@@ -118,12 +128,14 @@ static GOptionEntry general_options[] = {
     N_("Don't use pango markup language in dialog's text"), NULL },
   { "no-escape", 0, 0, G_OPTION_ARG_NONE, &options.data.no_escape,
     N_("Don't close dialog if Escape was pressed"), NULL },
+  { "escape-ok", 0, 0, G_OPTION_ARG_NONE, &options.data.escape_ok,
+    N_("Escape acts like OK button"), NULL },
   { "borders", 0, 0, G_OPTION_ARG_INT, &options.data.borders,
     N_("Set window borders"), N_("NUMBER") },
   { "always-print-result", 0, 0, G_OPTION_ARG_NONE, &options.data.always_print,
     N_("Always print result"), NULL },
   { "response", 0, 0, G_OPTION_ARG_INT, &options.data.def_resp,
-    N_("Set default response for Ctrl+Enter"), N_("NUMBER") },
+    N_("Set default return code"), N_("NUMBER") },
   { "selectable-labels", 0, 0, G_OPTION_ARG_NONE, &options.data.selectable_labels,
     N_("Dialog text can be selected"), NULL },
   /* window settings */
@@ -156,12 +168,10 @@ static GOptionEntry general_options[] = {
   { "tabnum", 0, 0, G_OPTION_ARG_INT, &options.tabnum,
     N_("Tab number of this dialog"), N_("NUMBER") },
 #ifndef G_OS_WIN32
-  { "parent-win", 0, 0, G_OPTION_ARG_INT, &options.parent,
-    N_("XID of parent window"), "XID" },
   { "kill-parent", 0, G_OPTION_FLAG_OPTIONAL_ARG, G_OPTION_ARG_CALLBACK, parse_signal,
-    N_("Send SIGNAL to parent"), N_("SIGNAL") },
-  { "print-xid", 0, 0, G_OPTION_ARG_NONE, &options.print_xid,
-    N_("Print X Window Id to the stderr"), NULL },
+    N_("Send SIGNAL to parent"), N_("[SIGNAL]") },
+  { "print-xid", 0, G_OPTION_FLAG_OPTIONAL_ARG, G_OPTION_ARG_CALLBACK, set_xid_file,
+    N_("Print X Window Id to the file/stderr"), N_("[FILENAME]") },
 #endif
   { NULL }
 };
@@ -181,6 +191,8 @@ static GOptionEntry common_options[] = {
     N_("Set item separator character"), N_("SEPARATOR") },
   { "editable", 0, 0, G_OPTION_ARG_NONE, &options.common_data.editable,
     N_("Allow changes to text in some cases"), NULL },
+  { "tail", 0, 0, G_OPTION_ARG_NONE, &options.common_data.tail,
+    N_("Autoscroll to end of text"), NULL },
   { "quoted-output", 0, 0, G_OPTION_ARG_NONE, &options.common_data.quoted_output,
     N_("Quote dialogs output"), NULL },
   { "num-output", 0, 0, G_OPTION_ARG_NONE, &options.common_data.num_output,
@@ -201,6 +213,10 @@ static GOptionEntry common_options[] = {
     N_("Identifier of embedded dialogs"), N_("KEY") },
   { "complete", 0, 0, G_OPTION_ARG_CALLBACK, set_complete_type,
     N_("Set extended completion for entries (any, all, or regex)"), N_("TYPE") },
+#if GLIB_CHECK_VERSION(2,30,0)
+  { "iec-format", 0, G_OPTION_FLAG_OPTIONAL_ARG, G_OPTION_ARG_CALLBACK, set_size_format,
+    N_("Use IEC (base 1024) units with for size values"), NULL },
+#endif
 #ifdef HAVE_SPELL
   { "enable-spell", 0, 0, G_OPTION_ARG_NONE, &options.common_data.enable_spell,
     N_("Enable spell check for text"), NULL },
@@ -236,7 +252,7 @@ static GOptionEntry color_options[] = {
   { "gtk-palette", 0, 0, G_OPTION_ARG_NONE, &options.color_data.gtk_palette,
     N_("Show system palette in color dialog"), NULL },
   { "palette", 0, G_OPTION_FLAG_OPTIONAL_ARG, G_OPTION_ARG_CALLBACK, add_palette,
-    N_("Set path to palette file. Default - " RGB_FILE), N_("FILENAME") },
+    N_("Set path to palette file. Default - " RGB_FILE), N_("[FILENAME]") },
   { "expand-palette", 0, 0, G_OPTION_ARG_NONE, &options.color_data.expand_palette,
     N_("Expand user palette"), NULL },
   { "mode", 0, 0, G_OPTION_ARG_CALLBACK, set_color_mode,
@@ -253,6 +269,8 @@ static GOptionEntry dnd_options[] = {
     N_("Display drag-n-drop box"), NULL },
   { "tooltip", 0, 0, G_OPTION_ARG_NONE, &options.dnd_data.tooltip,
     N_("Use dialog text as tooltip"), NULL },
+  { "exit-on-drop", 0, 0, G_OPTION_ARG_INT, &options.dnd_data.exit_on_drop,
+    N_("Exit after NUMBER of drops"), N_("NUMBER") },
   { NULL }
 };
 
@@ -340,6 +358,12 @@ static GOptionEntry html_options[] = {
     N_("Set mime type of input stream data"), N_("TYPE") },
   { "encoding", 0, 0, G_OPTION_ARG_STRING, &options.html_data.encoding,
     N_("Set encoding of input stream data"), N_("ENCODING") },
+  { "uri-handler", 0, 0, G_OPTION_ARG_STRING, &options.html_data.uri_cmd,
+    N_("Set external handler for clicked uri"), N_("CMD") },
+  { "user-agent", 0, 0, G_OPTION_ARG_STRING, &options.html_data.user_agent,
+    N_("Set user agent string"), N_("STRING") },
+  { "user-style", 0, 0, G_OPTION_ARG_STRING, &options.html_data.user_style,
+    N_("Set path or uri to user styles"), "URI" },
   { NULL }
 };
 #endif
@@ -426,6 +450,8 @@ static GOptionEntry list_options[] = {
     N_("Use regex in search"), NULL },
   { "no-selection", 0, 0, G_OPTION_ARG_NONE, &options.list_data.no_selection,
     N_("Disable selection"), NULL },
+  { "add-on-top", 0, 0, G_OPTION_ARG_NONE, &options.list_data.add_on_top,
+    N_("Add new records on the top of a list"), NULL },
   { NULL }
 };
 
@@ -457,6 +483,8 @@ static GOptionEntry notebook_options[] = {
     N_("Set position of a notebook tabs (top, bottom, left or right)"), N_("TYPE") },
   { "tab-borders", 0, 0, G_OPTION_ARG_INT, &options.notebook_data.borders,
     N_("Set tab borders"), N_("NUMBER") },
+  { "active-tab", 0, 0, G_OPTION_ARG_INT, &options.notebook_data.active,
+    N_("Set active tab"), N_("NUMBER") },
   { NULL }
 };
 
@@ -469,6 +497,8 @@ static GOptionEntry notification_options[] = {
     N_("Disable exit on middle click"), NULL },
   { "hidden", 0, 0, G_OPTION_ARG_NONE, &options.notification_data.hidden,
     N_("Doesn't show icon at startup"), NULL },
+  { "icon-size", 0, 0, G_OPTION_ARG_INT, &options.notification_data.icon_size,
+    N_("Set icon size for fully specified icons (default - 16)"), N_("SIZE") },
   { NULL }
 };
 
@@ -521,7 +551,7 @@ static GOptionEntry progress_options[] = {
   { "rtl", 0, 0, G_OPTION_ARG_NONE, &options.progress_data.rtl,
     N_("Right-To-Left progress bar direction"), NULL },
   { "enable-log", 0, G_OPTION_FLAG_OPTIONAL_ARG, G_OPTION_ARG_CALLBACK, set_progress_log,
-    N_("Show log window"), N_("TEXT") },
+    N_("Show log window"), N_("[TEXT]") },
   { "log-expanded", 0, 0, G_OPTION_ARG_NONE, &options.progress_data.log_expanded,
     N_("Expand log window"), NULL },
   { "log-on-top", 0, 0, G_OPTION_ARG_NONE, &options.progress_data.log_on_top,
@@ -570,8 +600,6 @@ static GOptionEntry text_options[] = {
     N_("Set justification (left, right, center or fill)"), N_("TYPE") },
   { "margins", 0, 0, G_OPTION_ARG_INT, &options.text_data.margins,
     N_("Set text margins"), N_("SIZE") },
-  { "tail", 0, 0, G_OPTION_ARG_NONE, &options.text_data.tail,
-    N_("Autoscroll to end of text"), NULL },
   { "show-cursor", 0, G_OPTION_FLAG_REVERSE, G_OPTION_ARG_NONE, &options.text_data.hide_cursor,
     N_("Show cursor in read-only mode"), NULL },
   { "show-uri", 0, 0, G_OPTION_ARG_NONE, &options.text_data.uri,
@@ -585,6 +613,8 @@ static GOptionEntry text_options[] = {
 static GOptionEntry source_options[] = {
   { "lang", 0, 0, G_OPTION_ARG_STRING, &options.source_data.lang,
     N_("Use specified langauge for syntax highlighting"), N_("LANG") },
+  { "theme", 0, 0, G_OPTION_ARG_STRING, &options.source_data.theme,
+    N_("Use specified theme"), N_("THEME") },
   { NULL }
 };
 #endif
@@ -595,7 +625,7 @@ static GOptionEntry filter_options[] = {
   { "mime-filter", 0, 0, G_OPTION_ARG_CALLBACK, add_file_filter,
     N_("Sets a mime-type filter"), N_("NAME | MIME1 MIME2 ...") },
   { "image-filter", 0, G_OPTION_FLAG_OPTIONAL_ARG, G_OPTION_ARG_CALLBACK, add_file_filter,
-    N_("Add filter for images"), N_("NAME") },
+    N_("Add filter for images"), N_("[NAME]") },
   { NULL }
 };
 
@@ -604,6 +634,14 @@ static GOptionEntry misc_options[] = {
     N_("Show about dialog"), NULL },
   { "version", 0, 0, G_OPTION_ARG_NONE, &version_mode,
     N_("Print version"), NULL },
+#ifdef HAVE_SPELL
+  { "show-langs", 0, 0, G_OPTION_ARG_NONE, &langs_mode,
+    N_("Show list of spell languages"), NULL },
+#endif
+#ifdef HAVE_SOURCEVIEW
+  { "show-themes", 0, 0, G_OPTION_ARG_NONE, &themes_mode,
+    N_("Show list of GtkSourceView themes"), NULL },
+#endif
   { "gtkrc", 0, 0, G_OPTION_ARG_FILENAME, &options.gtkrc_file,
     N_("Load additional GTK settings from file"), N_("FILENAME") },
   { "hscroll-policy", 0, 0, G_OPTION_ARG_CALLBACK, set_scroll_policy,
@@ -1157,7 +1195,23 @@ set_scroll_policy (const gchar * option_name, const gchar * value, gpointer data
     options.vscroll_policy = pt;
 }
 
+#if GLIB_CHECK_VERSION(2,30,0)
+static gboolean
+set_size_format (const gchar * option_name, const gchar * value, gpointer data, GError ** err)
+{
+  options.common_data.size_fmt = G_FORMAT_SIZE_IEC_UNITS;
+}
+#endif
+
 #ifndef G_OS_WIN32
+static gboolean
+set_xid_file (const gchar * option_name, const gchar * value, gpointer data, GError ** err)
+{
+  options.print_xid = TRUE;
+  if (value && value[0])
+    options.xid_file = g_strdup (value);
+}
+
 static gboolean
 parse_signal (const gchar * option_name, const gchar * value, gpointer data, GError ** err)
 {
@@ -1304,6 +1358,14 @@ yad_set_mode (void)
     options.mode = YAD_MODE_ABOUT;
   else if (version_mode)
     options.mode = YAD_MODE_VERSION;
+#ifdef HAVE_SPELL
+  else if (langs_mode)
+    options.mode = YAD_MODE_LANGS;
+#endif
+#ifdef HAVE_SOURCEVIEW
+  else if (themes_mode)
+    options.mode = YAD_MODE_THEMES;
+#endif
 }
 
 void
@@ -1315,9 +1377,9 @@ yad_options_init (void)
   options.extra_data = NULL;
   options.gtkrc_file = NULL;
 #ifndef G_OS_WIN32
-  options.parent = 0;
   options.kill_parent = 0;
   options.print_xid = FALSE;
+  options.xid_file = NULL;
 #endif
 
   options.hscroll_policy = GTK_POLICY_AUTOMATIC;
@@ -1351,6 +1413,7 @@ yad_options_init (void)
   options.data.borders = 2;
   options.data.no_markup = FALSE;
   options.data.no_escape = FALSE;
+  options.data.escape_ok = FALSE;
   options.data.always_print = FALSE;
   options.data.selectable_labels = FALSE;
   options.data.def_resp = YAD_RESPONSE_OK;
@@ -1376,6 +1439,7 @@ yad_options_init (void)
   options.common_data.item_separator = "!";
   options.common_data.multi = FALSE;
   options.common_data.editable = FALSE;
+  options.common_data.tail = FALSE;
   options.common_data.command = NULL;
   options.common_data.date_format = settings.date_format;
   options.common_data.float_precision = 3;
@@ -1389,6 +1453,9 @@ yad_options_init (void)
   options.common_data.filters = NULL;
   options.common_data.key = -1;
   options.common_data.complete = YAD_COMPLETE_SIMPLE;
+#if GLIB_CHECK_VERSION(2,30,0)
+  options.common_data.size_fmt = G_FORMAT_SIZE_DEFAULT;
+#endif
 #ifdef HAVE_SPELL
   options.common_data.enable_spell = FALSE;
   options.common_data.spell_lang = NULL;
@@ -1413,6 +1480,7 @@ yad_options_init (void)
 
   /* Initialize DND data */
   options.dnd_data.tooltip = FALSE;
+  options.dnd_data.exit_on_drop = 0;
 
   /* Initialize entry data */
   options.entry_data.entry_text = NULL;
@@ -1453,6 +1521,9 @@ yad_options_init (void)
   options.html_data.print_uri = FALSE;
   options.html_data.mime = NULL;
   options.html_data.encoding = NULL;
+  options.html_data.uri_cmd = NULL;
+  options.html_data.user_agent = "YAD-Webkit (" VERSION ")";
+  options.html_data.user_style = NULL;
 #endif
 
   /* Initialize icons data */
@@ -1495,6 +1566,7 @@ yad_options_init (void)
   options.list_data.regex_search = FALSE;
   options.list_data.clickable = TRUE;
   options.list_data.no_selection = FALSE;
+  options.list_data.add_on_top = FALSE;
 
   /* Initialize multiprogress data */
   options.multi_progress_data.bars = NULL;
@@ -1504,11 +1576,13 @@ yad_options_init (void)
   options.notebook_data.tabs = NULL;
   options.notebook_data.borders = 5;
   options.notebook_data.pos = GTK_POS_TOP;
+  options.notebook_data.active = 1;
 
   /* Initialize notification data */
   options.notification_data.middle = TRUE;
   options.notification_data.hidden = FALSE;
   options.notification_data.menu = NULL;
+  options.notification_data.icon_size = 16;
 
   /* Initialize paned data */
   options.paned_data.orient = GTK_ORIENTATION_VERTICAL;
@@ -1555,14 +1629,13 @@ yad_options_init (void)
   options.text_data.wrap = FALSE;
   options.text_data.justify = GTK_JUSTIFY_LEFT;
   options.text_data.margins = 0;
-  options.text_data.tail = FALSE;
-  options.text_data.uri = FALSE;
   options.text_data.hide_cursor = TRUE;
   options.text_data.uri_color = "blue";
 
 #ifdef HAVE_SOURCEVIEW
-  /* Initialize text data */
+  /* Initialize sourceview data */
   options.source_data.lang = NULL;
+  options.source_data.theme = NULL;
 #endif
 }
 
