@@ -14,7 +14,7 @@
  * You should have received a copy of the GNU General Public License
  * along with YAD. If not, see <http://www.gnu.org/licenses/>.
  *
- * Copyright (C) 2008-2016, Victor Ananjevsky <ananasik@gmail.com>
+ * Copyright (C) 2008-2017, Victor Ananjevsky <ananasik@gmail.com>
  */
 
 #include <sys/types.h>
@@ -199,7 +199,7 @@ raw_print_done (GtkPrintJob * job, gint * ret, GError * err)
       g_printerr (_("Printing failed: %s\n"), err->message);
       *ret = 1;
     }
-  gtk_main_quit ();
+  yad_exit (options.data.def_resp);
 }
 
 static void
