@@ -14,7 +14,7 @@
  * You should have received a copy of the GNU General Public License
  * along with YAD. If not, see <http://www.gnu.org/licenses/>.
  *
- * Copyright (C) 2008-2016, Victor Ananjevsky <ananasik@gmail.com>
+ * Copyright (C) 2008-2017, Victor Ananjevsky <ananasik@gmail.com>
  */
 
 #include <stdio.h>
@@ -119,7 +119,7 @@ handle_stdin (GIOChannel * channel, GIOCondition condition, gpointer data)
                     {
                       gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (progress_bar), 1.0);
                       if (options.progress_data.autoclose && options.plug == -1)
-                        yad_exit (YAD_RESPONSE_OK);
+                        yad_exit (options.data.def_resp);
                     }
                   else
                     gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (progress_bar), percentage / 100.0);
@@ -142,7 +142,7 @@ handle_stdin (GIOChannel * channel, GIOCondition condition, gpointer data)
         }
 
       if (options.progress_data.autoclose && options.plug == -1)
-        yad_exit (YAD_RESPONSE_OK);
+        yad_exit (options.data.def_resp);
 
       g_io_channel_shutdown (channel, TRUE, NULL);
       return FALSE;
@@ -194,6 +194,9 @@ progress_create_widget (GtkWidget * dlg)
       sw = gtk_scrolled_window_new (NULL, NULL);
       gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (sw), GTK_SHADOW_ETCHED_IN);
       gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (sw), options.hscroll_policy, options.vscroll_policy);
+#if GTK_CHECK_VERSION(3,22,0)
+      gtk_scrolled_window_set_propagate_natural_height (GTK_SCROLLED_WINDOW (sw), TRUE);
+#endif
       gtk_container_add (GTK_CONTAINER (ex), sw);
 
       progress_log = gtk_text_view_new ();
